package com.install4j.api.beaninfo;

/**
 * Base class for bean infos of actions classes.
 * <p>Using this class is not strictly required. In principle, you could also set values for the <tt>ATTRIBUTE_*</tt> constants in the bean descriptor
 * of an unrelated bean info class.
 */
public abstract class ActionBeanInfo extends Install4JBeanInfo {

    /**
     * @see #setAssociatedConfirmation(String, String)
     */
    public static final String ATTRIBUTE_ASSOCIATED_CONFIRMATION = "associatedConfirmation";

    /**
     * @see #setAssociatedConfirmation(String, String)
     */
    public static final String ATTRIBUTE_ASSOCIATED_CONFIRMATION_INIT_SCRIPT = "associatedConfirmationInitScript";

    /**
     * @see #setAssociatedScreen(String)
     */
    public static final String ATTRIBUTE_ASSOCIATED_SCREEN = "associatedScreen";

    /**
     * @see #setInstallerTargetScreen(String)
     */
    public static final String ATTRIBUTE_INSTALLER_TARGET_SCREEN = "preferredTargetInstallerScreen";

    /**
     * @see #setUninstallerTargetScreen(String)
     */
    public static final String ATTRIBUTE_UNINSTALLER_TARGET_SCREEN = "preferredTargetUninstallerScreen";

    /**
     * @see #setAssociatedStartupAction(String, boolean)
     */
    public static final String ATTRIBUTE_ASSOCIATED_STARTUP_ACTION = "associatedStartupAction";

    /**
     * @see #ATTRIBUTE_ASSOCIATED_STARTUP_ACTION_INSTALLER_AND_UNINSTALLER
     */
    public static final String ATTRIBUTE_ASSOCIATED_STARTUP_ACTION_INSTALLER_AND_UNINSTALLER = "associatedStartupActionInstallerAndUninstaller";

    /**
     * @see #ATTRIBUTE_DEFAULT_FAILURE_STRATEGY
     */
    public static final String ATTRIBUTE_DEFAULT_FAILURE_STRATEGY = "defaultFailureStrategy";

    /**
     * @see #ATTRIBUTE_DEFAULT_ERROR_MESSAGE
     */
    public static final String ATTRIBUTE_DEFAULT_ERROR_MESSAGE = "defaultErrorMessage";

    /**
     * Same as {@link Install4JBeanInfo#Install4JBeanInfo(String, String, String, boolean, boolean, Integer, Class, Class)}
     */
    protected ActionBeanInfo(String displayName, String shortDescription, String category, boolean multipleInstancesSupported, boolean installedFilesRequired, Integer sortKey, Class beanClass, Class customizerClass) {
        super(displayName, shortDescription, category, multipleInstancesSupported, installedFilesRequired, sortKey, beanClass, customizerClass);
    }

    /**
     * Same as {@link #ActionBeanInfo(String, String, String, boolean, boolean, Integer, Class, Class)} with a customizer class of <tt>null</tt>.
     */
    protected ActionBeanInfo(String displayName, String shortDescription, String category, boolean multipleInstancesSupported, boolean installedFilesRequired, Integer sortKey, Class beanClass) {
        super(displayName, shortDescription, category, multipleInstancesSupported, installedFilesRequired, sortKey, beanClass);
    }


    /**
     * Configures an associated confirmation. If the user adds the action, the install4j GUI will ask the user whether to add
     * a question to the associated screen, if the screen is a form screen or to the "Additional confirmation" screen
     * otherwise. The form screen will receive a check box form component with a variable name equal to the simple
     * class name of the action. The condition expression of the action will be set to <tt>context.getBooleanVariable("variableName")</tt>
     * so that the confirmation and the execution of the action are automatically wired together.
     * <p>If the <tt>initScript</tt> parameter is not <tt>null</tt> and the user accepts the addition of the confirmation,
     * the "Initialization script" property of the check box form component is set to <tt>initScript</tt>.
     * @param confirmationDescription the message of the confirmation
     * @param initScript the init script
     * @see #ATTRIBUTE_ASSOCIATED_CONFIRMATION
     * @see #ATTRIBUTE_ASSOCIATED_CONFIRMATION_INIT_SCRIPT
     */
    public void setAssociatedConfirmation(String confirmationDescription, String initScript) {
        if (confirmationDescription != null) {
            getBeanDescriptor().setValue(ATTRIBUTE_ASSOCIATED_CONFIRMATION, confirmationDescription);
        }
        if (initScript != null) {
            getBeanDescriptor().setValue(ATTRIBUTE_ASSOCIATED_CONFIRMATION_INIT_SCRIPT, initScript);
        }
    }

    /**
     * Configures an associated screen. If set, and the user adds the action, the install4j GUI checks if
     * an instance of the specified class is present in the list of configured screens. If not, it will ask the user whether to add
     * such a screen.
     * @param className the class name of the associated screen
     * @see #ATTRIBUTE_ASSOCIATED_SCREEN
     */
    public void setAssociatedScreen(String className) {
        if (className != null) {
            getBeanDescriptor().setValue(ATTRIBUTE_ASSOCIATED_SCREEN, className);
        }
    }

    /**
     * Configures a target screen for the installer. If set, and the user adds the action, the install4j GUI checks
     * if the current screen is an instance of the specified class. If not, it will ask the user whether the action
     * should rather be added to the target screen, adding that screen if it is not present in the list of configured screens.
     * @param className the class name of the target screen
     * @see #ATTRIBUTE_INSTALLER_TARGET_SCREEN
     */
    public void setInstallerTargetScreen(String className) {
        if (className != null) {
            getBeanDescriptor().setValue(ATTRIBUTE_INSTALLER_TARGET_SCREEN, className);
        }
    }

    /**
     * Configures a target screen for the uninstaller. If set, and the user adds the action, the install4j GUI checks
     * if the current screen is an instance of the specified class. If not, it will ask the user whether the action
     * should rather be added to the target screen, adding that screen if it is not present in the list of configured screens.
     * @param className the class name of the target screen
     * @see #ATTRIBUTE_UNINSTALLER_TARGET_SCREEN
     */
    public void setUninstallerTargetScreen(String className) {
        if (className != null) {
            getBeanDescriptor().setValue(ATTRIBUTE_UNINSTALLER_TARGET_SCREEN, className);
        }
    }

    /**
     * Configures an associated startup action. If set, and the user adds the action, the install4j GUI checks
     * if the startup screen contains an action of the specified class. If not, it will ask the user whether the action
     * of that class should be added to the startup screen.
     * @param className the class name of the associated startup action
     * @param installerAndUninstaller if the startup action should be added for both the installer and uninstaller or for the installer only
     * @see #ATTRIBUTE_ASSOCIATED_STARTUP_ACTION
     * @see #ATTRIBUTE_ASSOCIATED_STARTUP_ACTION_INSTALLER_AND_UNINSTALLER
     */
    public void setAssociatedStartupAction(String className, boolean installerAndUninstaller) {
        if (className != null) {
            getBeanDescriptor().setValue(ATTRIBUTE_ASSOCIATED_STARTUP_ACTION, className);
            getBeanDescriptor().setValue(ATTRIBUTE_ASSOCIATED_STARTUP_ACTION_INSTALLER_AND_UNINSTALLER, installerAndUninstaller ? Boolean.TRUE : Boolean.FALSE);
        }
    }

    /**
     * Configures the default failure strategy for the action.
     * @param failureStrategy the default failure strategy
     * @see #ATTRIBUTE_DEFAULT_FAILURE_STRATEGY
     */
    public void setDefaultFailureStrategy(FailureStrategy failureStrategy) {
        if (failureStrategy != null) {
            getBeanDescriptor().setValue(ATTRIBUTE_DEFAULT_FAILURE_STRATEGY, failureStrategy);
        }
    }

    /**
     * Configures the default error message for the action.
     * @param errorMessage the default error message
     * @see #ATTRIBUTE_DEFAULT_ERROR_MESSAGE
     */
    public void setDefaultErrorMessage(String errorMessage) {
        if (errorMessage != null) {
            getBeanDescriptor().setValue(ATTRIBUTE_DEFAULT_ERROR_MESSAGE, errorMessage);
        }
    }


}
