package com.install4j.api.context;

import com.install4j.api.actions.Action;
import com.install4j.api.beans.Bean;
import com.install4j.api.beans.ExternalFile;
import com.install4j.api.beans.ScriptProperty;
import com.install4j.api.events.InstallerEventListener;
import com.install4j.api.screens.Screen;

import java.io.File;
import java.util.Collection;
import java.util.MissingResourceException;
import java.util.Set;

/**
 * Base class for installation and uninstallation context.
 * The actual context passed to <a href="../screens/package-summary.html">screens</a>,
 * <a href="../actions/package-summary.html">actions</a> and
 * <a href="../formcomponents/package-summary.html">form components</a>
 * is either an {@link InstallerContext} or an {@link UninstallerContext}, depending on
 * whether they are part of the installer or uninstaller.
 *
 * @author ej-technologies GmbH
 * @see InstallerContext
 * @see UninstallerContext
 */
public interface Context {

    /**
     * Return the ISO code for the language that the installer is running with. If only the
     * principal language has been configured for the installer, this will always be the code for
     * the principal language. If additional languages have been configured, the user can
     * select a language or the language has been automatically determined by the installer, depending
     * on the settings on the "Languages" tab in the install4j GUI. If the installer chooses
     * a language automatically and none of the additional languages matches the system locale, the
     * principal language will be used.
     *
     * @return the ISO code for the language. This code is displayed in brackets after the language name in the
     *         install4j GUI.
     */
    String getLanguageId();

    /**
     * Get a message for a localization key.
     * The key can be either a system key as present in the <tt>resource/messages/messages_*.utf8</tt> files
     * or a key in a custom localization file.
     * <p>The search order is
     * <ul>
     * <li>custom localization file for actual language
     * <li>custom localization file for principal language
     * <li>system localization file provided by install4j
     * </ul>
     *
     * @param key the localization key
     * @return the message
     * @throws MissingResourceException if the key cannot be found
     */
    String getMessage(String key) throws MissingResourceException;

    /**
     * Retrieves the installation directory selected by the user, or the default
     * installation directory if the user has not yet made a selection.
     *
     * @return the installation directory
     */
    File getInstallationDirectory();

    /**
     * Returns whether the installer or uninstaller is running in unattended mode.
     *
     * @return <tt>true</tt> or <tt>false</tt>.
     */
    boolean isUnattended();

    /**
     * Returns whether the installer or uninstaller is running in console mode.
     *
     * @return <tt>true</tt> or <tt>false</tt>.
     */
    boolean isConsole();

    /**
     * Returns the application id of this project.
     *
     * @return the application id
     */
    String getApplicationId();

    /**
     * Returns the add on application id if this project generates add-on installers.
     *
     * @return the add on application id or the empty string if the project generates regular installers.
     */
    String getAddOnApplicationId();

    /**
     * Add a listener to receive installer events.
     *
     * @param listener the listener that should be added.
     */
    void addInstallerEventListener(InstallerEventListener listener);

    /**
     * Remove a listener added with <tt>addInstallerEventListener</tt>.
     *
     * @param listener the listener that should be removed.
     * @see #addInstallerEventListener(com.install4j.api.events.InstallerEventListener)
     */
    void removeInstallerEventListener(InstallerEventListener listener);

    /**
     * Programmatically switch to another screen in the installer. If
     * the given screen has not been registered, the operation will
     * fail. Registered screens can be obtained from the
     * <tt>getScreenById(...)</tt>, <tt>getScreens(...)</tt> and <tt>getFirstScreen(...)</tt> methods.
     * <p><b>Note:</b> This method only has a direct effect when called from a screen. When called from an action,
     * all remaining actions for the screen will be executed and the most recent request for a screen change will then be performed.
     *
     * @param screen the screen to be activated.
     * @see #getScreens()
     * @see #getScreens(Class)
     * @see #getFirstScreen(Class)
     */
    void gotoScreen(Screen screen);

    /**
     * Get a particular screen with the specified ID. You can display screen IDs in the install4j IDE.
     *
     * @param id the ID of the screen
     * @return the screens, or <tt>null</tt> if no screen with the spcified ID exists
     */
    Screen getScreenById(String id);

    /**
     * Get all registered screens.
     *
     * @return the screens
     */
    Screen[] getScreens();

    /**
     * Get all registered screens for a given screen class.
     *
     * @param screenClass the class of the screen.
     * @return the screens or <tt>null</tt> if no screen of this class could be found.
     */
    Screen[] getScreens(Class screenClass);

    /**
     * Get the registered screen for a given screen class. If multiple screens of the same
     * class are registered, the first occurrence is returned. In that case,
     * <tt>getScreens(Class screenClass)</tt> should be used.
     *
     * @param screenClass the class of the screen.
     * @return the screen or <tt>null</tt> if no screen of this class could be found.
     * @see #getScreens(Class)
     */
    Screen getFirstScreen(Class screenClass);

    /**
     * Get a particular action with the specified ID. You can display action IDs in the install4j IDE.
     *
     * @param id the ID of the action
     * @return the action, or <tt>null</tt> if no action with the spcified ID exists
     */
    Action getActionById(String id);

    /**
     * Get all registered actions for a specified screen.
     *
     * @param screen the screen whose associated actions should be returned
     * @return the actions or <tt>null</tt> if the specified screen has no associated actions.
     */
    Action[] getActions(Screen screen);

    /**
     * Get all registered actions for a specified screen and a given action class.
     *
     * @param actionClass the class of the action.
     * @param screen      the screen whose associated actions should be returned
     * @return the actions or <tt>null</tt> if no actions of this class could be found for the specified screen.
     */
    Action[] getActions(Class actionClass, Screen screen);

    /**
     * Get the registered action for a specified screen and a given action class. If multiple actions of the same
     * class are registered, the first occurrence is returned. In that case,
     * <tt>getActions(Class actionClass, Screen screen)</tt> should be used.
     *
     * @param actionClass the class of the action.
     * @param screen      the screen whose associated action should be returned or <tt>null</tt> if all screens should be searched
     * @return the action or <tt>null</tt> if no action of this class could be found for the specified screen.
     * @see #getActions(Class, com.install4j.api.screens.Screen)
     */
    Action getFirstAction(Class actionClass, Screen screen);

    /**
     * Get the wizard context if the installer or uninstaller is running in GUI mode.
     * The wizard context allows you to modify the installer GUI.
     *
     * @return the wizard context or <tt>null</tt> if the installer or uninstaller is running in unattended or console mode.
     */
    WizardContext getWizardContext();

    /**
     * Returns all additional files sets contained in this installer. If no file sets
     * have been defined in the install4j GUI, the returned collection will be empty.
     * The default file set is not contained in this list and will always be installed
     * by the "Install files" action.
     * The items in the collection will be in the same order as defined in the GUI.
     *
     * @return a collection containing <tt>FileSet</tt> instances.
     * @see FileSetSetup
     */
    Collection getFileSets();

    /**
     * Returns the file set for a specified ID. You can display
     * file set IDs in the install4j IDE.
     *
     * @param id the ID
     * @return the <tt>FileSet</tt> or <tt>null</tt> if the ID doesn't exist.
     * @see FileSetSetup
     */
    FileSetSetup getFileSetById(String id);

    /**
     * Returns all installation components for this installer. If no installation components
     * have been defined in the install4j GUI, the returned collection will be empty.
     * The items in the collection will be in the same order as defined in the GUI.
     *
     * @return a collection containing <tt>InstallationComponentSetup</tt> instances.
     * @see InstallationComponentSetup
     */
    Collection getInstallationComponents();

    /**
     * Returns the installation component for a specified ID. You can display
     * component IDs in the install4j IDE.
     *
     * @param id the ID
     * @return the <tt>InstallationComponentSetup</tt> or <tt>null</tt> if the ID doesn't exist.
     * @see InstallationComponentSetup
     */
    InstallationComponentSetup getInstallationComponentById(String id);

    /**
     * Returns all launchers for this installer. If no launchers
     * have been defined in the install4j GUI, the returned collection will be empty.
     * The items in the collection will be in the same order as defined in the GUI.
     *
     * @return a collection containing <tt>LauncherSetup</tt> instances.
     * @see LauncherSetup
     */
    Collection getLaunchers();

    /**
     * Returns the launcher for a specified ID. The ID of a launcher
     * can be displayed the install4j GUI by invoking Launcher->Show Launcher IDs from the main
     * menu when the Launchers section is active. The method is important for beans that
     * have a property of type <tt>java.lang.String</tt> whose property descriptor
     * has a context of type <tt>CONTEXT_LAUNCHER_ID</tt>. The property will contain a launcher
     * id that can be resolved at runtime with this method.
     *
     * @param id the ID
     * @return the <tt>LauncherSetup</tt> or <tt>null</tt> if the ID doesn't exist.
     * @see com.install4j.api.beaninfo.Install4JPropertyDescriptor#CONTEXT_LAUNCHER_ID
     */
    LauncherSetup getLauncherById(String id);

    /**
     * Returns whether the user has cancelled the installation or not. Long-running actions
     * should check this method frequently to comply with a cancel request from the user.
     *
     * @return <tt>true</tt> or <tt>false</tt>.
     */
    boolean isCancelling();

    /**
     * If you encounter an exception in your code that cannot be handled, you can use this method to invoke
     * the default exception handling of the install4j runtime. An error report will be prepared, the user will be
     * notified and the installer or uninstaller will quit. No rollback will be performed.
     * <p>This method is <b>for exceptional circumstances only</b>. Do not call this method if an action fails in an expected way:
     * if you return <tt>false</tt> from an action, its configurable failure strategy can be used to inform the user
     * of the failure and rollback previous actions.
     *
     * @param e the exception
     */
    void handleCriticalException(Throwable e);

    /**
     * Get the location where the installer will install a file in the distribution tree.
     * This method is important for actions with properties of type <tt>java.io.File</tt>.
     * The property values will be relative file names and contain encoded information
     * about installation roots. With this method, the installation location can be resolved.
     * Note that the result of this method can change over time as installer variables or the installation
     * directory are changed.
     *
     * @param archiveFile the relative file in the distribution tree
     * @return the location of the installed file
     */
    File getDestinationFile(File archiveFile);

    /**
     * Same as <tt>getDestinationFile</tt>, only with a path argument.
     *
     * @param archivePath the relative path in the distribution tree
     * @return the location of the installed file
     * @see #getDestinationFile(java.io.File)
     */
    File getDestinationFile(String archivePath);

    /**
     * Analyze a path in the distribution tree. Beans with <tt>java.io.File</tt> properties
     * can use this method to split paths into root information and relative information
     * and to resolve the installation location of a file in the distribution tree.
     * The <tt>getDestinationFile(..)</tt> methods directly give you the installed file location.
     *
     * @param archivePath the relative path in the distribution tree
     * @return the <tt>FileInfo</tt> information object
     * @see #getDestinationFile(java.io.File)
     * @see #getDestinationFile(String)
     */
    FileInfo getDestinationFileInfo(String archivePath);

    /**
     * Get an installer variable. In the install4j GUI, installer variables
     * are written as ${installer:variableName}. Please read the help topic on variables in the documentation
     * for more information.
     *
     * @param variableName the name of the installer variable.  Do not use the IDE syntax ${installer:...} for this parameter.
     * @return the variable value or <tt>null</tt> if the variable with the specified name was not defined.
     * @see #setVariable(String, Object)
     */
    Object getVariable(String variableName);

    /**
     * Convenience method to get an installer variable with a type of <tt>java.lang.Boolean</tt>.
     *
     * @param variableName the name of the installer variable. Do not use the IDE syntax ${installer:...} for this parameter.
     * @return <tt>true</tt> if the varaible value equals <tt>java.lang.Boolean.TUE</tt> or <tt>false</tt> otherwise
     *         (also if the variable has not been defined)
     * @see #getVariable(String)
     */
    boolean getBooleanVariable(String variableName);

    /**
     * Set an installer variable. Please read the help topic on variables in the documentation
     * for more information.
     *
     * @param variableName the name of the installer variable. Do not use the IDE syntax ${installer:...} for this parameter.
     * @param value        the new variable value
     * @see #getVariable(String)
     */
    void setVariable(String variableName, Object value);

    /**
     * Get the names of all defined installer variables. Please read the help topic on variables in the documentation.
     *
     * @return a set of variables names of type <tt>java.lang.String</tt>.
     * @see #getVariable(String)
     * @see #setVariable(String, Object)
     */
    Set getVariableNames();

    /**
     * Gets the value of a compiler variable as it was defined for this media file
     * at compile time. Compiler variables are defined on the "Compiler variables" tab
     * and can be overridden for each media file. System compiler variables are also available
     * from this method. In the install4j IDE, compiler variables
     * are written as ${compiler:variableName}. Please read the help topic on variables in the documentation
     * for more information.
     *
     * @param variableName the name of the compiler variable. Do not use the IDE syntax ${compiler:...} for this parameter.
     * @return the variable value or <tt>null</tt> if the variable with the specified name was not defined.
     */
    String getCompilerVariable(String variableName);

    /**
     * Run a script. Beans that have properties of type <tt>com.install4j.api.beans.ScriptProperty</tt>
     * can run these user-defined scripts at runtime with this method. Script properties require
     * a property descriptor in the associated BeanInfo class that contains information about the return type
     * as well as any additional parameters.
     *
     * @param scriptProperty the value of the script property
     * @param bean           the bean that defines the script property
     * @param parameters     the additional parameters or <tt>null</tt> if there are no additional parameters
     * @return the result of the script or <tt>null</tt> if the retun type of the script is <tt>void</tt>
     * @throws Exception Scripts are user-defined and can throw all sorts of exceptions.
     * @see com.install4j.api.beaninfo.ScriptPropertyDescriptor
     */
    Object runScript(ScriptProperty scriptProperty, Bean bean, Object[] parameters) throws Exception;

    /**
     * Get the current progress interface. If called by a screen or form component, this will be a default progress
     * interface where status messages and progress methods do nothing. A screen can replace that default progress
     * interface with a different implementation so that the status messages and progress methods invoked by the associated
     * actions are displayed on the screen. This is done by the "Installation" screen and the customizable
     * "Display progress" screen.
     * <p>During rollback, a special rollback dialog will be shown and make its progress interface available through
     * this method.
     *
     * @return the current progress interface
     * @see Screen#getProgressInterface(ProgressInterface)
     */
    ProgressInterface getProgressInterface();

    /**
     * Go forward in the screen sequence.
     * <p><b>Note:</b> This method only has a direct effect when called from a screen. When called from an action,
     * all remaining actions for the screen will be executed and the most recent request for a screen change will then be performed.
     *
     * @param numberOfScreens the number of screens to move forward. If the resulting index is beyond the last screen, the
     *                        installer or uninstaller will quit.
     * @param checkCondition  if the "Condition expression" of the target screen should be checked or not
     * @param executeActions  if the associated actions of the current screen should be performed or not. This parameter
     *                        has no effect if this method is called from an action.
     */
    void goForward(int numberOfScreens, boolean checkCondition, boolean executeActions);

    /**
     * Go backwards in the screen sequence. This is the sequence that is displayed on the "Screens" tab in the install4j GUI.
     * You can go backwards in the screen history with the <tt>goBackInHistory</tt> method.
     * <p><b>Note:</b> This method only has a direct effect when called from a screen. When called from an action,
     * all remaining actions for the screen will be executed and the most recent request for a screen change will then be performed.
     *
     * @param numberOfScreens the number of screens to move backwards. If the resulting index is negative, a
     *                        <tt>RuntimeException</tt> will be thrown.
     * @see #goBackInHistory(int)
     */
    void goBack(int numberOfScreens);

    /**
     * Go backwards in the screen history. This is not necessarily the same as the screen sequence, since the screenflow can be
     * changed programmatically.
     * <p><b>Note:</b> This method only has a direct effect when called from a screen. When called from an action,
     * all remaining actions for the screen will be executed and the most recent request for a screen change will then be performed.
     *
     * @param numberOfScreens the number of screens to move backwards in the screen history. If the resulting index is negative, a
     *                        <tt>RuntimeException</tt> will be thrown.
     * @see #goBack(int)
     */
    void goBackInHistory(int numberOfScreens);

    /**
     * Go backwards in the screen history to the target screen. If
     * the given screen has not been registered or was not display previously, the operation will
     * fail. Registered screens can be obtained from the
     * <tt>getScreenById(...)</tt>, <tt>getScreens(...)</tt> and <tt>getFirstScreen(...)</tt> methods.
     * <p><b>Note:</b> This method only has a direct effect when called from a screen. When called from an action,
     * all remaining actions for the screen will be executed and the most recent request for a screen change will then be performed.
     *
     * @param targetScreen the screen to be activated.
     * @see #getScreens()
     * @see #getScreens(Class)
     * @see #getFirstScreen(Class)
     */
    void goBackInHistory(Screen targetScreen);

    /**
     * Get the location of an external file that was packaged with the installer. Beans that define properties of type <tt>ExternalFile</tt>
     * can access those files at runtime. When the installer is started, an external file
     * is located in a temporary directory. After the "Install files" action has run, the external file is
     * also copied to the <tt>.install4j</tt> directory in the installation directory.
     *
     * @param externalFile      the value of the <tt>ExternalFile</tt> property
     * @param installedLocation if <tt>true</tt>, this method returns the location in the <tt>.install4j</tt> directory,
     *                          otherwise the location in the temporary directory is returned. If called in an uninstaller, this argument has no effect and
     *                          the location in the <tt>.install4j</tt> directory will always be returned.
     * @return the resolved external file
     */
    File getExternalFile(ExternalFile externalFile, boolean installedLocation);

    /**
     * Determine if an error has occured for any of the previous actions. After an action fails, this method
     * returns <tt>true</tt>. If the failure strategy of the failed action is configured in such a way that
     * the execution of the installer continues, you might want to react to the error in a subsequent action.
     * <p><b>Note:</b> This error flag is never cleared. You have to clear it manually with the <tt>setErrorOccured</tt>
     * method. If you want to make sure that the error originates in a certain range of actions, you have to add
     * a "Run script" action just before that range and call <tt>context.setErrorOccured(false)</tt> in order to
     * clear any earlier error.
     *
     * @return if an error has occurred or not
     * @see #setErrorOccured(boolean)
     */
    boolean isErrorOccured();

    /**
     * Clear or set the error flag that is described for <tt>isErrorOccured</tt>.
     *
     * @param errorOccured the new error flag
     * @see #isErrorOccured()
     */
    void setErrorOccured(boolean errorOccured);

    /**
     * Register a variable that should be written to the response file when the installation is finished.
     *
     * @param variableName the variable name
     */
    void registerResponseFileVariable(String variableName);

    /**
     * Register a variable whose values should not be written to the installation log file.
     * You can use this method for variables which contain passwords or other information that
     * should not be saved to disk.
     *
     * @param variableName the variable name
     */
    void registerHiddenVariable(String variableName);


    /**
     * Triggers a reboot at the end of installation or uninstallation on Windows.
     *
     * @param askUser if true, the user will be asked if he wants to perform a reboot.
     */
    void triggerReboot(boolean askUser);
}
