package com.install4j.api.context;

import java.io.File;

/**
 * The user notification interface during installation. It displays progress
 * and messages and can ask questions relevant for installation purposes
 * in custom actions.
 * <p>Status messages and progress are only displayed if the progress
 * interface is used by an action and the containing screen offers its own progress interface,
 *  like the "Installation" screen or the customizable "Display progress" screen.
 * @author ej-technologies GmbH
 * @see com.install4j.api.screens.Screen#getProgressInterface(ProgressInterface)
 * @see Context#getProgressInterface()
 */
public interface ProgressInterface {

    /**
     * Returned by <tt>askOverwrite</tt> if the user hits "No".
     */
    public static final int OVERWRITE_NO = 1;

    /**
     * Returned by <tt>askOverwrite</tt> if the user hits "Yes".
     */
    public static final int OVERWRITE_YES = 2;

    /**
     * Returned by <tt>askOverwrite</tt> if the user hits "All".
     */
    public static final int OVERWRITE_ALL = 3;

    /**
     * Returned by <tt>askOverwrite</tt> if the user hits "Never".
     */
    public static final int OVERWRITE_NEVER = 4;

    /**
     * Returned by <tt>askRetry</tt> if the user hits "No".
     */
    public static final int RETRY_NO = 1;

    /**
     * Returned by <tt>askRetry</tt> if the user hits "Yes".
     */
    public static final int RETRY_YES = 2;

    /**
     * Returned by <tt>askRetry</tt> if the user hits "Cancel".
     */
    public static final int RETRY_CANCEL = 3;

    /**
     * Sets the status message on the progress screen.
     * <p>Note: Has no effect if not called from an action or if the containing screen doesn't display progress.
     * @param message the new message.
     */
    public void setStatusMessage(String message);

    /**
     * Sets the detail message on the progress screen. This message is shown
     * below the status message.
     * <p>Note: Has no effect if not called from an action or if the containing screen doesn't display progress.
     * @param message the new message.
     */
    public void setDetailMessage(String message);

    /**
     * Sets the percentage of completion of the current action.
     * <p>Note: Has no effect if not called from an action or if the containing screen doesn't display progress.
     * @param value a value between 0 and 100.
     */
    public void setPercentCompleted(int value);

    /**
     * Retrieves the current progress value
     * @return the current progress value between 0 and 100.
     *  Always returns 0 if not called from an action or if the containing screen doesn't display progress.
     */
    public int getPercentCompleted();

    /**
     * Indicate that an action is performed whose duration is not known.
     * <p>Note: Has no effect if not called from an action or if the containing screen doesn't display progress.
     * @param indeterminateProgress pass <tt>true</tt> to switch on indeterminate progress
     *        feedback and <tt>false</tt> to switch it off again. When you switch it off,
     *        the previously set percentage will be shown in the progess bar.
     */
    public void setIndeterminateProgress(boolean indeterminateProgress);

    /**
     * Shows an error dialog with the specified message.
     * @param message the error message
     */
    public void showFailure(String message);

    /**
     * Asks the user whether the specified file should be overwritten.
     * @param file the file in question.
     * @return one of the <tt>OVERWRITE_*</tt> constants.
     * @throws UserCanceledException if the user cancels the question
     */
    public int askOverwrite(File file) throws UserCanceledException;

    /**
     * Asks the user whether the installer should retry to overwrite the
     * specified file.
     * @param file the file in question.
     * @return one of the <tt>RETRY_*</tt> constants
     * @throws UserCanceledException if the user cancels the question
     */
    public int askRetry(File file) throws UserCanceledException;

    /**
     * Asks the user whether to continue after the specified file could not
     * be installed.
     * @param file the file in question.
     * @return <tt>true</tt> if the user wants to continue the installation.
     * @throws UserCanceledException if the user cancels the question
     */
    public boolean askContinue(File file) throws UserCanceledException;

}
